;;;; dicemusic.lisp
;;;
;;; 23sep19 - mp : initial
;;;
;;; Generate music in the manner of an 18th century dice game.
;;;
;;; NOTE: requires lilypond to create score
;;;
(in-package #:dicemusic)

(defparameter *progname* "Waltzermatic")
(defparameter *piece-title* "Eine Kleine Waltzer"
  "Default title of the piece")
(defparameter *output-dir* "output/"
  "Where the generated output goes")

;; ---------------------( structures )-----
;;
(defparameter *mozart-waltz*
  '((( 96  99 141  41 105 122  11  30)  ;;  2 First Part
     ( 32   6 128  63 146  46 134  81)  ;;  3
     ( 69  93 158  13 153  35 110  24)  ;;  4
     ( 40  17 113  83 161   9 159 100)  ;;  5
     (148  74 163  43  80  97  36 107)  ;;  6
     (104 157  97 167 154  68 118  91)  ;;  7
     (159  60 171  53  99 133  21 127)  ;;  8
     (119  84 114  50 140  86 169  94)  ;;  9
     ( 98 142  42 136  75 129  62 123)  ;; 10
     (  3  87 163  61 135  47 147  33)  ;; 11
     ( 54 130  10 103  28  37 106   4)) ;; 12
    (( 70 121  26   9 112  49 109  14)  ;;  2 Second Part
     (117  39 126  36 174  18 116  83)  ;;  3
     ( 66 139  13 132  75  48 143  79)  ;;  4
     ( 90 176   7  34  67 160 52  170)  ;;  5
     ( 25 143  64 125  76 136  1   93)  ;;  6
     (138  71 140  29 101 162  23 141)  ;;  7
     ( 16 135  47 175  43 168  89 172)  ;;  8
     (120  88  48 166  51 115  72 111)  ;;  9
     ( 65  77  19  82 137  38 149   8)  ;; 10
     (109   4  31 164 144  59 173  78)  ;; 11
     ( 83  20 108  92  19 124  44 131)));; 12
  "The lookup table for the Mozart (Simrock) waltzes")


;; ---------------------( utility functions )-----
;;
;; NOTE: the following two functions are taken from
;; the Alexandia library package: hash-tables.lisp.
;;
(defun maphash-keys (function table)
  "Like MAPHASH, but calls FUNCTION with each key in the hash table TABLE."
  (maphash (lambda (k v)
             (declare (ignore v))
             (funcall function k))
           table))

(defun hash-table-keys (table)
  "Returns a list containing the keys of the hash table"
  (let ((keys nil))
    (maphash-keys (lambda (k)
                    (push k keys))
                  table)
    keys))

(defun random-fname (&key (ext "txt"))
  "Quick-n-dirty generation of a random file name"
   (format nil "~X-~X.~a" (random 65535) (random 4294967295) ext))

(defun dice (&key (n 1) (sides 6))
  "Simulate the roll of n dice, returning the results in a list"
  (make-random-state)
  (loop for i from 1 to n
       collect (+ (random sides) 1)))

(defun roll2d6 (&key (n 1))
  "Roll 2 six-sided dice and sum the result, n times"
  (loop for i from 1 to n
       collect (apply #'+ (dice :n 2 :sides 6))))

;; ---------------------( working fns )------

(defun fragment-num (iter roll part &key (table *mozart-waltz*))
  "Given a iteration, dice roll, and part, return the fragment number"
  (nth iter (nth (- roll 2) (nth part table))))


(defun piece-seq (&key (table *mozart-waltz*) (bars 7))
  "Generate the sequence of musical fragments, bars long"
  (list
   (loop for i from 0 to bars
      collect (fragment-num i (first (roll2d6)) 0 :table table))
   (loop for i from 0 to bars
	collect (fragment-num i (first (roll2d6)) 1 :table table))))

;; -------------------( lilypond stuff )-------
;;
;; For the Mozart Waltzer, there are a max of 3 voices:
;; one in the right (treble), and one or two in the bass.
;; The time signature is 3/8 and the key C (with various
;; accidentals as required). There are 176 bar fragments.
;;
;; The structure will be based on using parallel voices.
;;
(defun lp-score-header (&key (outs t) (title *piece-title*))
  "Print the name and title of the score"
  (format outs "\\\version \"2.18.2\"~%")
  (format outs "\\header {~%")
  (format outs "  title = \"~a\"~%" title)
  (format outs "  composer = \"~a\"~%" *progname*)
  (format outs "}~%"))

(defun lp-score-global (&key (outs t))
  "Preparing the score"
  (format outs "global = {~%")
  (format outs "  \\\key c \\\major~%")
  (format outs "  \\\time 3/8~%")
  (format outs "}~%"))

(defun lp-score-bars-in (&key (outs t))
  "Begin our bars"
  (format outs "\\\parallelMusic #'(voiceA voiceB voiceC) {~%"))

(defun lp-score-bars-out (&key (outs t))
  "End our bars"
  (format outs "}~%"))

(defun lp-score-internals (&key (outs t) )
  "Internals of the lilypond score block"
  (format outs "  \\\new PianoStaff <<~%")
  (format outs "    \\\new Staff { ~%")
  (format outs "      \\\global~%")
  (format outs "      <<~%")
  (format outs "        \\\voiceA~%")
  (format outs "      >>~%")
  (format outs "    }~%")
  (format outs "    \\\new Staff { ~%")
  (format outs "      \\\global \\\clef bass~%")
  (format outs "      <<~%")
  (format outs "        \\\voiceB~%")
  (format outs "        \\\\~%")
  (format outs "        \\\voiceC~%")
  (format outs "      >>~%")
  (format outs "    }~%")
  (format outs "  >>~%"))

(defun lp-score-main (&key (outs t) (title "Waltzer"))
  "The main (non-midi) score block"
  (format outs "\\\score {~%")
  (lp-score-internals :outs outs)
  (format outs "  \\\layout { }~%")
  (format outs "  \\\header { piece = \"~a\" }~%" title)  
  (format outs "}~%"))

(defun lp-score-midi (&key (outs t))
  "The midi score block, with repeats"
  (format outs "\\\score {~%")
  (format outs "  \\unfoldRepeats { ~%")
  (lp-score-internals :outs outs)
  (format outs "  }~%")
  (format outs "  \\\midi { }~%")
  (format outs "}~%"))

(defun gen-bars (seq &key (outs t))
  "Print a musical fragment"
  (dolist (bar seq)
    (dolist (frag (gethash bar *mwf*))
      (format outs "    ")   ; for pretty
      (format outs frag))))

(defun gen-seq (seq &key (outs t) (repeats 2))
  "Print out the sequence of music fragments"
  (dolist (sect seq)
    (format outs "  \\\repeat volta ~d {~%" repeats)
    (gen-bars sect :outs outs)
    (format outs "  }~%")))

(defun gen-score (seq &key (outs t) (midi nil)
			(title *piece-title*) (subtitle nil))
  "Generate score from the given sequence, producing optional midi"
  (lp-score-header :outs outs :title title)
  (lp-score-global :outs outs)
  (lp-score-bars-in :outs outs)
  (gen-seq seq :outs outs)
  (lp-score-bars-out :outs outs)
  (if subtitle
      (lp-score-main :outs outs :title subtitle)
      (lp-score-main :outs outs :title seq))
  (if midi
      (lp-score-midi :outs outs)))


(defun gen-score-to-file (seq fname &key (midi nil)
				      (title *piece-title*)
				      (subtitle nil))
  "Generate a score and write it to a file"
  (with-open-file
      (stream (merge-pathnames *output-dir* fname)
	      :direction :output
	      :if-exists :supersede)
    (gen-score seq :outs stream :midi midi
	       :title title :subtitle subtitle)))

;;; ----------------( main )----------
;;;
(defun create-score ()
  "An easy to use top-level score generator"
  (let ((fname (concatenate 'string *progname* (random-fname :ext "ly")) ))
    (gen-score-to-file (piece-seq) fname :midi t )
    (format t "Printed score to file: ~a~a~%" *output-dir* fname)))


(defun dump-fragments (&key (fname "fragments.ly"))
  "Dump all of the musical fragments to a file"
  (gen-score-to-file (list (reverse (hash-table-keys *mwf*))) fname
		     :title "Waltzer Fragments" :subtitle "All Fragments")
  (format t "Printed fragments to file: ~a~a~%" *output-dir* fname))

;; -----------------( music fragments )-----
;; We store the music as entries in a hash table
;; indexed by an integer index. Each entry is a
;; format state, one for each voice.
;;
(defparameter *mwf* (make-hash-table)
  "Hash table for Mozart Waltz Fragments")

(setf (gethash 1 *mwf*)
      '("% bar 001~%"
	"f''8 d'' g'' |~%"
	"f8   d   g   |~%"
	"s4.          |~%"))

(setf (gethash 2 *mwf*)
      '("% bar 002             ~%"
	"a'8 fis'16 g' b' g'' |~%"
	"<b, g>4  e8\\\rest   |~%"
	"s4.                  |~%"))

(setf (gethash 3 *mwf*)
      '("% bar 003~%"
	"g''8 c'' e'' |~%"
	"<c e>4    e8\\\rest  |~%"
	"s4.          |~%"))

(setf (gethash 4 *mwf*)
      '("% bar 004~%"
	"g''8 d''4\\trill |~%"
	"g,16 b, g8 b, |~%"
	"s4.           |~%"))

(setf (gethash 5 *mwf*)
      '("% bar 005~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 6 *mwf*)
      '("% bar 006~%"
	"g'8  c'' e'' |~%"
	"<c e>4   e8\\\rest |~%"
	"s4.          |~%"))

(setf (gethash 7 *mwf*)
      '("% bar 007~%"
	"e''16 c'' e'' g'' c''' g'' |~%"
	"<c g>4        e8\\\rest    |~%"
	"s4.                        |~%"))

(setf (gethash 8 *mwf*)
      '("% bar 008~%"
	"c''4    r8  |~%"
	"c8   g, c,  |~%"
	"s4.         |~%"))

(setf (gethash 9 *mwf*)
      '("% bar 009~%"
	"<e'' c''>8 <d'' b'> r8  |~%"
	"g4                  g,8 |~%"
	"s4.          |~%"))

(setf (gethash 10 *mwf*)
      '("% bar 010~%"
	"b'16 a' b' c'' d'' b' |~%"
	"g4          e8\\\rest |~%"
	"s4.                   |~%"))

(setf (gethash 11 *mwf*)
      '("% bar 011~%"
	"e''16 c'' b' a' g' fis' |~%"
	"c8        d     c,      |~%"
	"s4.                     |~%"))

(setf (gethash 12 *mwf*)
      '("% bar 012~%"
	"<e' c'>8 <e' c'> <e' c'> |~%"
	"c8      c      c      |~%"
	"s4.         |~%"))

(setf (gethash 13 *mwf*)
      '("% bar 013           ~%"
	"c''8( g' e')       |~%"
	"<e g>4   e8\\\rest |~%"
	"s4.                |~%"))

(setf (gethash 14 *mwf*)  ; NB identical to 008
      '("% bar 014~%"
	"c''4    r8  |~%"
	"c8   g, c,  |~%"
	"s4.         |~%"))

(setf (gethash 15 *mwf*)
      '("% bar 015           ~%"
	"e''8 g''16 e'' c''8 |~%"
	"<c g>4  <c e>8      |~%"
	"s4.                 |~%"))

(setf (gethash 16 *mwf*)
      '("% bar 016           ~%"
	"a''8( fis'' d'')   |~%"
	"<d fis>4  <c fis>8 |~%"
	"s4.                |~%"))

(setf (gethash 17 *mwf*)
      '("% bar 017             ~%"
	"c''16 g' c'' e'' g' c'' |~%"
	"<e g>4          e8\\\rest |~%"
	"s4.                   |~%"))

(setf (gethash 18 *mwf*)
      '("% bar 018           ~%"
	"g'8( c'' e'') |~%"
	"<c e>4 <c g>8 |~%"
	"s4.           |~%"))

(setf (gethash 19 *mwf*)
      '("% bar 019             ~%"
	"e''16 c'' e''8 g'' |~%"
	"<c g>4    <c e>8   |~%"
	"s4.                |~%"))

(setf (gethash 20 *mwf*)
      '("% bar 020             ~%"
	"g''8 b''16 d''' d''8 |~%"
	"b,4       e8\\rest   |~%"
	"s4.                  |~%"))

(setf (gethash 21 *mwf*)
      '("% bar 021~%"
	"c''16 e'' g'' d'' a' fis'' |~%"
	"c8        d     c,         |~%"
	"s4.                        |~%"))

(setf (gethash 22 *mwf*)
      '("% bar 022           ~%"
	"e''8 c'' g'       |~%"
	"c4      e8\\\rest |~%"
	"s4.               |~%"))

(setf (gethash 23 *mwf*)
      '("% bar 023~%"
	"f''16 e'' d'' e'' f'' g'' |~%"
	"f16   e   b,  e   f   g   |~%"
	"s4.                       |~%"))

(setf (gethash 24 *mwf*)  ; NB same as 005
      '("% bar 024~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 25 *mwf*)
      '("% bar 025~%"
	"d'16 fis' a' d'' fis'' a'' |~%"
	"d4               c8        |~%"
	"s4.                   |~%"))

(setf (gethash 26 *mwf*)
      '("% bar 026~%"
	"<c'' e''>8 <c'' e''> <c'' e''> |~%"
	"c16  e      g   e     c'  c    |~%"
	"s4.                   |~%"))

(setf (gethash 27 *mwf*)
      '("% bar 027~%"
	"f''16 e'' f'' d'' c'' b' |~%"
	"<g b>4         e8\\\rest |~%"
	"s4.                   |~%"))

(setf (gethash 28 *mwf*)
      '("% bar 028~%"
	"fis''16 d'' a' a'' fis'' d'' |~%"
	"<c a>4         e8\\\rest |~%"
	"s4.                   |~%"))

(setf (gethash 29 *mwf*)
      '("% bar 029~%"
	"b'16 d'' g'' d'' b'8 |~%"
	"g4               g,8 |~%"
	"s4.                  |~%"))

(setf (gethash 30 *mwf*)  ; NB same as 005
      '("% bar 030~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 31 *mwf*)
      '("% bar 031~%"
	"e''16 c'' g'8 e'' |~%"
	"<c g>4    <c g>8  |~%"
	"s4.               |~%"))

(setf (gethash 32 *mwf*)
      '("% bar 32 ~%"
        "g'8 c'' e''  |~%"
        "<c e>4  e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 33 *mwf*)  ; NB same as 005
      '("% bar 033~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 34 *mwf*)
      '("% bar 34 ~%"
        "e''16 c'' d'' b' g'8 |~%"
        "g4  e8\\\rest       |~%"
        "s4.    |~%"))

(setf (gethash 35 *mwf*)
      '("% bar 35 ~%"
        "a'8 d'' fis''  |~%"
        "<d fis>4 <c a>8 |~%"
        "s4.    |~%"))

(setf (gethash 36 *mwf*)
      '("% bar 36 ~%"
        "a'16 e'' d'' g'' fis'' a'' |~%"
        "c8       d       c,        |~%"
        "s4.    |~%"))

(setf (gethash 37 *mwf*)
      '("% bar 37 ~%"
        "g''16 c''' g'' d'' b'8 |~%"
        "<b d>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 38 *mwf*)
      '("% bar 38 ~%"
        "c''8 g' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 39 *mwf*)
      '("% bar 39 ~%"
        "g''8 g' g' |~%"
        "b,16 d g d b, g, |~%"
        "s4.    |~%"))

(setf (gethash 40 *mwf*)
      '("% bar 40 ~%"
        "c''16 b' c'' e'' g' c'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 41 *mwf*)
      '("% bar 41 ~%"
        "c''16 b' b'' e'' g'8 |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 42 *mwf*)
      '("% bar 42 ~%"
        "b'16 c'' d'' b' a' g' |~%"
        "g,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 43 *mwf*)
      '("% bar 43 ~%"
        "g''8 f''16 e'' d'' c'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 44 *mwf*)
      '("% bar 44 ~%"
        "a'8 f''16d'' a' b' |~%"
        "f4           g8 |~%"
        "s4.    |~%"))

(setf (gethash 45 *mwf*)
      '("% bar 45 ~%"
        "c''16 b' c'' g' e' c' |~%"
        "<e g>4   e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 46 *mwf*)
      '("% bar 46 ~%"
        "g''8 c'''16 g'' d'' b' |~%"
        "<b, d>4   e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 47 *mwf*)
      '("% bar 47 ~%"
        "g''8 g''16 d'' c'''8 |~%"
        "<b, d>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 48 *mwf*)
      '("% bar 48 ~%"
        "e''8 c''16 e'' g'' c''' |~%"
        "<c g>4 <c e>8 |~%"
        "s4.    |~%"))

(setf (gethash 49 *mwf*)
      '("% bar 49 ~%"
        "e''8( c'' g') |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 50 *mwf*)
      '("% bar 50 ~%"
        "c''8 e''16 c'' g'8 |~%"
        "<e g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 51 *mwf*)
      '("% bar 51 ~%"
        "c''16 g' e'' c'' g'' e'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 52 *mwf*)
      '("% bar 52 ~%"
        "d''16( cis'') d'' f'' g' b' |~%"
        "f4        g8 |~%"
        "s4.    |~%"))

(setf (gethash 53 *mwf*)
      '("% bar 53 ~%"
        "<c'' e''>8 <c'' e''>16 <d'' f''> <e'' g''>8 |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 54 *mwf*)
      '("% bar 54 ~%"
        "<e' c''>8 <e' c''> <e' c''> |~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 55 *mwf*)
      '("% bar 55 ~%"
        "g''8 b'' d'' |~%"
        "<b, d>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 56 *mwf*)
      '("% bar 56 ~%"
        "d''16 b' g'8 r8 |~%"
        "<g, g>4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 57 *mwf*)
      '("% bar 57 ~%"
        "e''8 c'' g' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 58 *mwf*)
      '("% bar 58 ~%"
        "g''8 e'' c'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 59 *mwf*)
      '("% bar 59 ~%"
        "g''8 c'' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 60 *mwf*)
      '("% bar 60 ~%"
        "g''8 f''16 e'' d'' c'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 61 *mwf*)
      '("% bar 61 ~%"
        "c''8 e''16 c'' g''8 |~%"
        "<e g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 62 *mwf*)
      '("% bar 62 ~%"
        "e''16 c'' b' g' a' fis' |~%"
        "c8 d c, |~%"
        "s4.    |~%"))

(setf (gethash 63 *mwf*)
      '("% bar 63 ~%"
        "e''16 c'' b' c'' g'8 |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 64 *mwf*)
      '("% bar 64 ~%"
        "e''16 g'' c''' g'' e'' c'' |~%"
        "<c g>4 <c g>8 |~%"
        "s4.    |~%"))

(setf (gethash 65 *mwf*)
      '("% bar 65 ~%"
        "d''16 a' d''8 fis'' |~%"
        "fis4 e8\\\rest |~%"
        "d4   s8 |~%"))

(setf (gethash 66 *mwf*)
      '("% bar 66 ~%"
        "fis''8 a'' fis'' |~%"
        "a8     fis d     |~%"
        "d8     d   c     |~%"))

(setf (gethash 67 *mwf*)
      '("% bar 67 ~%"
        "c''16 b' c'' e'' g' c'' |~%"
        "e4 <e g>8 |~%"
        "c4 s8     |~%"))

(setf (gethash 68 *mwf*)
      '("% bar 68 ~%"
        "g''8 b''16 g'' d'' g'' |~%"
        "b,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 69 *mwf*)
      '("% bar 69 ~%"
        "g''8( e'' c'') |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 70 *mwf*)
      '("% bar 70 ~%"
        "fis''8 a''16 fis'' d'' fis'' |~%"
        "d4 c8 |~%"
        "s4.    |~%"))

(setf (gethash 71 *mwf*)
      '("% bar 71 ~%"
        "g''16 b'' d''' b'' g''8 |~%"
        "<b, d>4 <b, d>8 |~%"
        "s4.    |~%"))

(setf (gethash 72 *mwf*)
      '("% bar 72 ~%"
        "f''16 e'' d'' c'' b' d'' |~%"
        "f4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 73 *mwf*)     ; NB same as 048
      '("% bar 73 ~%"
        "e''8 c''16 e'' g'' c''' |~%"
        "<c g>4 <c e>8 |~%"
        "s4.    |~%"))

(setf (gethash 74 *mwf*)
      '("% bar 74 ~%"
        "c'''16b'' c''' g'' e'' c'' |~%"
        "e4 e8\\\rest |~%"
        "c4 s8   |~%"))

(setf (gethash 75 *mwf*)
      '("% bar 75 ~%"
        "<d'' fis''>8 <d'' fis''> <d'' fis''> |~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 76 *mwf*)
      '("% bar 76 ~%"
        "c'''16 b'' c''' g'' e'' c'' |~%"
        "<c e>4 <c g>8 |~%"
        "s4.    |~%"))

(setf (gethash 77 *mwf*)
      '("% bar 77 ~%"
        "g''16 b'' g''8 d'' |~%"
        "<b, d>4 <b, g>8 |~%"
        "s4.    |~%"))

(setf (gethash 78 *mwf*)
      '("% bar 78 ~%"
        "c''8 c' r8  |~%"
        "c4      c,8 |~%"
        "s4.    |~%"))

(setf (gethash 79 *mwf*)
      '("% bar 79 ~%"
        "c''4 r8  |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 80 *mwf*)
      '("% bar 80 ~%"
        "d''8 a'\\turn fis'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 81 *mwf*)  ; NB same as 005
      '("% bar 081~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 82 *mwf*)
      '("% bar 82 ~%"
        "d''16 b' g'8 g'' |~%"
        "g4  d8  |~%"
        "b,4 b,8 |~%"))

(setf (gethash 83 *mwf*)
      '("% bar 83 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 84 *mwf*)
      '("% bar 84 ~%"
        "c''16 g' e'' c'' g'' e'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 85 *mwf*)
      '("% bar 85 ~%"
        "c''8 e'' g' |~%"
        "<e g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 86 *mwf*)
      '("% bar 86 ~%"
        "d''8 d''16 g'' b''8 |~%"
        "<b, g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 87 *mwf*)
      '("% bar 87 ~%"
        "g''8 c'' e'' |~%"
        "<c e>4 <c g>8 |~%"
        "s4.    |~%"))

(setf (gethash 88 *mwf*)
      '("% bar 88 ~%"
        "g''16 d'' g'' b'' g'' d'' |~%"
        "<b, d>4 <b, d>8 |~%"
        "s4.    |~%"))

(setf (gethash 89 *mwf*)
      '("% bar 89 ~%"
        "f''16 e'' d''8 g'' |~%"
        "f16   e   d8   g   |~%"
        "s4.    |~%"))

(setf (gethash 90 *mwf*)
      '("% bar 90 ~%"
        "fis''16 a'' d''' a'' fis'' a'' |~%"
        "<c a>4 <c a>8 |~%"
        "s4.    |~%"))

(setf (gethash 91 *mwf*)  ; NB same as 005
      '("% bar 091~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 92 *mwf*)
      '("% bar 92 ~%"
        "<b' d''>8 g''16 c''' d''8 |~%"
        "<g, g>4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 93 *mwf*)
      '("% bar 93 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 94 *mwf*)  ; NB same as 005
      '("% bar 094~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 95 *mwf*)
      '("% bar 95 ~%"
        "g''8 e'' c'' |~%"
        "e4 e8\\\rest |~%"
        "c4 s8    |~%"))

(setf (gethash 96 *mwf*)
      '("% bar 96 ~%"
        "e'' c'' g' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 97 *mwf*)
      '("% bar 97 ~%"
        "g''16 fis'' g'' d'' b' g' |~%"
        "<b, d>4 <b, g>8 |~%"
        "s4.    |~%"))

(setf (gethash 98 *mwf*)
      '("% bar 98 ~%"
        "c''8 g' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 99 *mwf*)
      '("% bar 99 ~%"
        "fis''8 a'' d'' |~%"
        "<c a>4 <c a>8 |~%"
        "s4.    |~%"))

(setf (gethash 100 *mwf*)  ; NB same as 005
      '("% bar 100~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 101 *mwf*)
      '("% bar 101 ~%"
        "e''16 d'' e'' g'' c''' g'' |~%"
        "g4 e8 |~%"
        "c4 c8 |~%"))

(setf (gethash 102 *mwf*)
      '("% bar 102 ~%"
        "fis''16 d'' a'8 fis'' |~%"
        "<c a>4 <c a>8 |~%"
        "s4.    |~%"))

(setf (gethash 103 *mwf*)
      '("% bar 103 ~%"
        "c''16 e'' c'' g' e'8 |~%"
        "<e g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 104 *mwf*)
      '("% bar 104 ~%"
        "e''16 d'' e'' g'' c''' g'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 105 *mwf*)
      '("% bar 105 ~%"
        "fis''8 a''16 fis'' d'' fis'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 106 *mwf*)
      '("% bar 106 ~%"
        "a'8 d''16 c'' b' a' |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 107 *mwf*)  ; NB same as 005
      '("% bar 107~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 108 *mwf*)
      '("% bar 108 ~%"
        "e''8( g'' c''') |~%"
        "<c g>4 <c e>8 |~%"
        "s4.    |~%"))

(setf (gethash 109 *mwf*)
      '("% bar 109 ~%"
        "d''16 f'' d'' f'' b' d'' |~%"
        "<f a>4 <g d'>8 |~%"
        "s4.    |~%"))

(setf (gethash 110 *mwf*)
      '("% bar 110 ~%"
        "<b' d''>16( <a' c''>) <a' c''>( <g' b'>) <g' b'>( <fis' a'>) |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 111 *mwf*)
      '("% bar 111 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 112 *mwf*)
      '("% bar 112 ~%"
        "e''8 c'' g' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 113 *mwf*)
      '("% bar 113 ~%"
        "fis''8 d'' b' |~%"
        "<g b>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 114 *mwf*)
      '("% bar 114 ~%"
        "<b' d''>8 <b' d''> <b' d''> |~%"
        "g8 g g |~%"
        "s4.    |~%"))

(setf (gethash 115 *mwf*)
      '("% bar 115 ~%"
        "c''16 g' e'' c'' g'' e'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 116 *mwf*)
      '("% bar 116 ~%"
        "d''16 f'' a'' f'' d'' b' |~%"
        "f4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 117 *mwf*)
      '("% bar 117 ~%"
        "d''16 a' d'' fis'' a'' fis'' |~%"
        "<d fis>4  e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 118 *mwf*)
      '("% bar 118 ~%"
        "e''16 a'' g'' b'' fis'' a'' |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 119 *mwf*)
      '("% bar 119 ~%"
        "e''16 c'' g'' e'' c''' g'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 120 *mwf*)
      '("% bar 120 ~%"
        "d'''8 a''16 fis'' d'' a' |~%"
        "<d fis>4 <c fis>8 |~%"
        "s4.    |~%"))

(setf (gethash 121 *mwf*)
      '("% bar 121 ~%"
        "g''8 b''16 g'' d''8 |~%"
        "<b, g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 122 *mwf*)
      '("% bar 122 ~%"
        "g''16 fis'' g'' b'' d''8 |~%"
        "<b d>8 <b d> <b g> |~%"
        "s4.    |~%"))

(setf (gethash 123 *mwf*)  ; NB same as 005
      '("% bar 123~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 124 *mwf*)
      '("% bar 124 ~%"
        "<e' c''>8 <e' c''> <e' c''> |~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 125 *mwf*)
      '("% bar 125 ~%"
        "g''16 e'' d'' b' g'8 |~%"
        "g8 g, e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 126 *mwf*)
      '("% bar 126 ~%"
        "c''16 g' c'' e'' g'' <c'' e''> |~%"
        "e4 e16 c |~%"
        "s4.    |~%"))

(setf (gethash 127 *mwf*)  ; NB same as 005
      '("% bar 127~%"
	"<g' b' d'' g''>4 r8 |~%"
	"g,8 b16 g  fis e |~%"
	"g,8 g16 f! e   d |~%"))

(setf (gethash 128 *mwf*)
      '("% bar 128 ~%"
        "b'8 d'' g'' |~%"
        "g,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 129 *mwf*)
      '("% bar 129 ~%"
        "a''16 g'' fis'' a'' d''8 |~%"
        "<b, d>8 <b, d> <b, g> |~%"
        "s4.    |~%"))

(setf (gethash 130 *mwf*)
      '("% bar 130 ~%"
        "<e' c''>8 <e' c''> <e' c''> |~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 131 *mwf*)
      '("% bar 131 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 132 *mwf*)
      '("% bar 132 ~%"
        "<c'' e''>8 <b' d''>16 <g' b'> g'8 |~%"
        "g8 g, e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 133 *mwf*)
      '("% bar 133 ~%"
        "d''8 g''16 d'' b' d'' |~%"
        "<b, g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 134 *mwf*)
      '("% bar 134 ~%"
        "a'16 e'' <b' d''> <a' c''> <g' b'> <fis' a'> |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 135 *mwf*)
      '("% bar 135 ~%"
        "fis''8 fis''16 d'' a''8 |~%"
        "d8 d d |~%"
        "c8 c c |~%"))

(setf (gethash 136 *mwf*)
      '("% bar 136 ~%"
        "c'''16 b'' c''' g'' e'' c'' |~%"
        "e4 e8\\\rest |~%"
        "c4 s8        |~%"))

(setf (gethash 137 *mwf*)   ; NB same as 098
      '("% bar 137 ~%"
        "c''8 g' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 138 *mwf*)
      '("% bar 138 ~%"
        "<a' d'' fis''>8 fis''4\\trill |~%"
        "d,16 d cis d c d |~%"
        "s4.    |~%"))

(setf (gethash 139 *mwf*)
      '("% bar 139 ~%"
        "g''16 c''' g'' c''' d''8 |~%"
        "b,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 140 *mwf*)
      '("% bar 140 ~%"
        "a'8 a'16 d'' fis''8 |~%"
        "<c fis>8 <c fis> <c a> |~%"
        "s4.    |~%"))

(setf (gethash 141 *mwf*)
      '("% bar 141 ~%"
        "d''16 e'' f'' d'' c'' b' |~%"
        "<b, g>4 g,8 |~%"
        "s4.    |~%"))

(setf (gethash 142 *mwf*)
      '("% bar 142 ~%"
        "c''8 g' e'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 143 *mwf*)
      '("% bar 143 ~%"
        "g''8 d''16 b' g'8 |~%"
        "<b, d>4 <b, d>8 |~%"
        "s4.    |~%"))

(setf (gethash 144 *mwf*) 
      '("% bar 144 ~%"
        "g''8 c'' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 145 *mwf*)
      '("% bar 145 ~%"
        "d''16 f'' a' d'' b' d'' |~%"
        "f4 g8  |~%"
        "s4.    |~%"))

(setf (gethash 146 *mwf*)
      '("% bar 146 ~%"
        "<fis' d''>8 <d'' fis''> <fis'' a''> |~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 147 *mwf*)
      '("% bar 147 ~%"
        "e''16 c''' b'' g'' a'' fis'' |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 148 *mwf*)
      '("% bar 148 ~%"
        "c'''16 b'' c''' g'' e'' c'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 149 *mwf*)
      '("% bar 149 ~%"
        "f''16 d'' a'8 b' |~%"
        "f4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 150 *mwf*)
      '("% bar 150 ~%"
        "<g' c'' e''>8 e''4\\trill |~%"
        "c16 b, c d e fis |~%"
        "s4.    |~%"))

(setf (gethash 151 *mwf*)
      '("% bar 151 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 152 *mwf*)
      '("% bar 152 ~%"
        "g''8 f''16 e'' d'' c'' |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 153 *mwf*)
      '("% bar 153 ~%"
        "d''16 a' fis'' d'' a'' fis'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 154 *mwf*)
      '("% bar 154 ~%"
        "d''16 cis'' d'' fis'' a'' fis'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 155 *mwf*)
      '("% bar 155 ~%"
        "g''16 b'' g'' d'' b' g' |~%"
        "<b, d>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 156 *mwf*)
      '("% bar 156 ~%"
        "c''16 g' e'' c'' g''8 |~%"
        "<e g>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 157 *mwf*)
      '("% bar 157 ~%"
        "e''16 d'' e'' g'' c''' g'' |~%"
        "c4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 158 *mwf*)
      '("% bar 158 ~%"
        "b'8 d''16 b' a' g' |~%"
        "g,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 159 *mwf*)
      '("% bar 159 ~%"
        "e''16 g'' d'' c'' b' a' |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 160 *mwf*)
      '("% bar 160 ~%"
        "c''16 b' c'' e'' g' c'' |~%"
        "<c e>4 <c e>8 |~%"
        "s4.    |~%"))

(setf (gethash 161 *mwf*)
      '("% bar 161 ~%"
        "<fis' d''>8 <fis' d''> <fis' d''>|~%"
        "c8 c c |~%"
        "s4.    |~%"))

(setf (gethash 162 *mwf*)
      '("% bar 162 ~%"
        "e''16 d'' e'' g'' c''' g'' |~%"
        "<c g>4 <c e>8 |~%"
        "s4.    |~%"))

(setf (gethash 163 *mwf*)
      '("% bar 163 ~%"
        "g''16 fis'' g'' d'' b' g' |~%"
        "<b, d>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 164 *mwf*)
      '("% bar 164 ~%"
        "d''8 g'4 |~%"
        "g16 fis g d b, g, |~%"
        "s4.    |~%"))

(setf (gethash 165 *mwf*)
      '("% bar 165 ~%"
        "d''8( b' g') |~%"
        "b,4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 166 *mwf*)
      '("% bar 166 ~%"
        "d''16 b'' g'' d'' b'8 |~%"
        "<g b>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 167 *mwf*)
      '("% bar 167 ~%"
        "c''8 c''16 d'' e''8 |~%"
        "<c e>4 e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 168 *mwf*)
      '("% bar 168 ~%"
        "g''8 f''16 e'' d'' c'' |~%"
        "<c e>4 <e g>8 |~%"
        "s4.    |~%"))

(setf (gethash 169 *mwf*)
      '("% bar 169 ~%"
        "e''16 g'' d'' g'' a' fis'' |~%"
        "c8 d d, |~%"
        "s4.    |~%"))

(setf (gethash 170 *mwf*)
      '("% bar 170 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 171 *mwf*)
      '("% bar 171 ~%"
        "b'16 c'' d'' e'' f'' d'' |~%"
        "<g, g>4 <b, g>8 |~%"
        "s4.    |~%"))

(setf (gethash 172 *mwf*)  ; NB same as 170
      '("% bar 172 ~%"
        "c''4 r8 |~%"
        "c8 g, c, |~%"
        "s4.    |~%"))

(setf (gethash 173 *mwf*)
      '("% bar 173 ~%"
        "f''16 a'' a'8 b'16 d'' |~%"
        "f4 g8 |~%"
        "s4.    |~%"))

(setf (gethash 174 *mwf*)
      '("% bar 174 ~%"
        "g'8 c'' e'' |~%"
        "<c e>16 g <c e> g <c e> g |~%"
        "s4.    |~%"))

(setf (gethash 175 *mwf*)
      '("% bar 175 ~%"
        "e''16 c'' b' d'' g''8 |~%"
        "g8 g, e8\\\rest |~%"
        "s4.    |~%"))

(setf (gethash 176 *mwf*)
      '("% bar 176 ~%"
        "a''16 g'' b'' g'' d'' g'' |~%"
        "<b, d>4 <b, d>8 |~%"
        "s4.    |~%"))

;;; ---------( the rest is silence )----------
